#include "lcd.h"
#include <avr/io.h>
#include <util/delay.h>
#include "functions.h"
#include <avr/interrupt.h>
#include <avr/eeprom.h>

//global vars
  uint16_t EEMEM eeKM;
  uint16_t EEMEM eeM;
  uint16_t EEMEM eeDKM;
  uint16_t EEMEM eeMaxTemp;
  uint16_t EEMEM eeWarnTemp;
  uint16_t EEMEM eeRadUmfang;  // in cm
  uint16_t EEMEM eeIgnitions;
  // normale Variablen
  uint16_t iWarnTemp;
  uint16_t iKM;
  uint16_t icm;
  uint16_t iM;
  uint16_t iMaxTemp;
  uint16_t iDKM;
  uint16_t iRadUmfang;
  uint16_t iIgnitions;
  uint16_t iRPM=0;
  uint16_t iV=0;
  uint16_t iTemp;
  uint16_t dzm_tmp=0;
  uint16_t tachosignale_tmp;
  uint16_t tacho_tmp;
  // ISR only data
  uint16_t tachoold=0;
  uint16_t tachonew=0;
  uint16_t dzmold=0;
  uint16_t dzmnew=0;
  // ISR - MAIN Variablen
  volatile uint16_t dzm_delta=0;
  volatile uint16_t tachosignale;
  volatile uint16_t tacho_delta;
  char scenario;
  char dzmflag;
  char lcdon=0;

ISR(TIMER0_OVF_vect) 
{
	TCNT0=246;	
	dzmnew=TCNT1;
	dzm_delta=dzmold-dzmnew;
	dzmold=dzmnew;
}

ISR(INT0_vect)
{
  tachosignale++;
  tachonew=TCNT1;
  tacho_delta=tachoold-tachonew;
  tachoold=tachonew;
}


ISR ( ANA_COMP_vect )
{
   eeprom_write_word(&eeKM, iKM);
  eeprom_write_word(&eeM, iM);
  eeprom_write_word(&eeDKM, iDKM);
  eeprom_write_word(&eeMaxTemp, iMaxTemp);
  lcd_bel(0);
  lcdon=0;
  lcd_clrscr();
  _delay_ms(5);
  lcd_puts("datasave");  
  lcd_bel(1);
  _delay_ms(2000);
  lcd_clrscr();
}


int main(void)
{

_delay_ms(1000);

//Portrichtungsregister setzen
  DDRA |= (1 << DDA1) | (1 << DDA2); //a0 data eingang, a1 cs ausg, a2 clk ausg 
  PORTA &= (1<<PA0); //internen pull up aktivieren
  DDRC |= (1<<PC7); //lcd bel pin
  DDRB  |= (1<<DDB0);// | (1<<DDB2) | (1<<DDB3);  //b0 als eingang fr dzm, b2,3 fr ac
  PORTB |= (1<<PB0) | (1<<PB2);    //internen pull up aktivieren
//  DDRD  |= (1<<DDD2);
  PORTD |= (1<<PD2);

//load var from eeprom
  iKM = eeprom_read_word(&eeKM);
  iM = eeprom_read_word(&eeM);
  iMaxTemp = eeprom_read_word(&eeMaxTemp);
  iDKM= eeprom_read_word(&eeDKM);
  iRadUmfang=eeprom_read_word(&eeRadUmfang);
  iIgnitions=eeprom_read_word(&eeIgnitions);
  iWarnTemp=eeprom_read_word(&eeWarnTemp);

//check if var is bogus and set 0 then
  if(iKM==0xFFFF){iKM=0;eeprom_write_word(&eeKM,0);}
  if(iM==0xFFFF){iM=0;eeprom_write_word(&eeM,0);}
  if(iMaxTemp==0xFFFF){iMaxTemp=0;eeprom_write_word(&eeMaxTemp,0);}
  if(iDKM==0xFFFF){iDKM=0;eeprom_write_word(&eeDKM,0);}
  if(iRadUmfang==0xFFFF){iRadUmfang=50;eeprom_write_word(&eeRadUmfang,50);}
  if(iIgnitions==0xFFFF){iIgnitions=1;eeprom_write_word(&eeIgnitions,1);}
  if(iWarnTemp==0xFFFF){iWarnTemp=1500;eeprom_write_word(&eeWarnTemp,1500);}

//displaybeleuchtung an
  lcd_bel(1);lcdon=1;
//zeit frs display aufzuwachen
  _delay_ms(50);
//lcd init
  _delay_ms(50);
  lcd_init(LCD_DISP_ON);
  _delay_ms(50);
  lcd_clrscr();
//welcome screen
  _delay_ms(50);    
  lcd_puts("   VesCom 1.0   ");
  _delay_ms(1000);
  lcd_gotoxy(0, 1);
  _delay_ms(5);
  lcd_puts("    by  dare");
  _delay_ms(3000);
  lcd_clrscr();  

//start timer for systemtime
   TCCR1B = 0x02;		// CLK_DIV=8

//set interrupt for timer overflow
  TIMSK |= (1<<TOV0);
//timer0 vorladen
  TCNT0=246;

//start timer for dzm, count on external clock
  TCCR0 = (1<<CS02) | (1<<CS01);  

//fallende Flanke lst interrupt aus
  MCUCR |= (1<<ISC11);
//enable interrupt on int0
  GIMSK |= (1<<INT0);

//startscenario
  scenario=1;

//set ac for power loose interrupt
//  ACSR = (1 << ACIE) | (1 << ACIS1); 


//activate interrupts
  sei();

// Endlose Hauptschleife

for(;;)
  {
  
//put temp in iTemp
    fReadTemp(&iTemp); 
//temp warning function
    if(iTemp>=iWarnTemp) 
    {
      if(lcdon==1)
      {
        lcd_bel(0); 
        lcdon=0;
      }else 
      {
        lcd_bel(1);
        lcdon=1;
      }
    } else lcd_bel(1);

  
//maxtemp
    if(iTemp>iMaxTemp){eeprom_write_word(&eeMaxTemp,iTemp);iMaxTemp=iTemp;}

//MENU ausgeben
    fOutput(scenario,iV,iRPM,iTemp,iMaxTemp,iKM,iM,iDKM); 

    if (fdebounce(&PINA, PA4)==1) {if(scenario>5)scenario=1; else scenario++; lcd_clrscr();}
    if (fdebounce(&PINA, PA3)==1) 
    {   
      eeprom_write_word(&eeKM, iKM);
      eeprom_write_word(&eeM, iM);
      eeprom_write_word(&eeDKM, iDKM);
      eeprom_write_word(&eeMaxTemp, iMaxTemp);
      menu(&eeDKM,&eeMaxTemp,&eeIgnitions,&eeWarnTemp,&eeRadUmfang);
    }
    
    // Daten der ISR kopiern, fr sptere Berechnungen
    // muss atomar sein
	    
    cli();
    tachosignale_tmp=tachosignale;
    tachosignale=0;
    tacho_tmp = tacho_delta;
	dzm_tmp = dzm_delta;			
    sei();

	// RPM berechnen
	// gemessen werden 10 Drehzahlpulse, somit Faktor 10*60s = 600
	// Quarztakt ist 4 MHz / 8 = 500.000
	// 500.000*600=300.000.000

	iRPM = 300000000/dzm_tmp;

	// Tachostand berechnen, Radumfang ist in cm
    // Workaround, weil 64 Bit Zahlen tierisch Flash fressen

	for (; tachosignale_tmp>0; tachosignale_tmp--) {
		icm+=iRadUmfang;
		if (icm>99) {
			icm-=100;
			iM++;
			if (iM>999) {
				iM=0;
				iKM++;
			}
		}
	}

	// Geschwindigkeit berechnen
	// gemessen wird 1 Tachopuls
	// Quarztakt ist 4 MHz / 8 = 500.000
	// m/s -> km/h Faktor 3.6
	// Radumfang in cm
	// 500.000*3,6/100=10800

	iRPM = (uint32_t)iRadUmfang*10800/tacho_tmp;
  }
return 0;
}
